package ru.amse.smyshlyaev.grapheditor.graphdatacalculator;

import java.awt.Graphics;

import ru.amse.smyshlyaev.grapheditor.graph.*;


public class VertexDataCalculator {
	public static final int MY_RADIUS = 20;
	public static final int MY_SPACE = 10;
	
	private static double getOvalRange(double x, double axe) {
		return 4 * x * x / axe / axe;
	}
	
	public static boolean isInVertex(Graphics gc, 
            			ISelectableVertex vertex, int x, int y) {
		return (getOvalRange(
		           (x - vertex.getX()),
		           getWidth(gc, vertex)
		    ) + 
		    getOvalRange (
				   (y - vertex.getY()),
				    getHeight(gc, vertex)
         )) < 1;
	}
	
	public static int getWidth(Graphics gc, ISelectableVertex vertex){
		return Math.max(
	               2 * MY_RADIUS, 
		    	   getLabelWidth(gc, vertex) + MY_SPACE
		   );
	}
	
	public static int getHeight(Graphics gc, ISelectableVertex vertex){
		return Math.max(
                2 * MY_RADIUS, 
                getLabelHeight(gc, vertex) + MY_SPACE
        );				
	}

	public static boolean isIntersection(Graphics gc, 
			ISelectableVertex start, ISelectableVertex end) {
		IEdge edge = new Edge(start, end);
		
		return isInVertex(
				gc,
	         	end,
				EdgeDataCalculator.getStartX(gc, edge), 
		    	EdgeDataCalculator.getStartY(gc, edge)
		   )|| isInVertex(
				   gc,
				   start,
				   EdgeDataCalculator.getEndX(gc, edge), 
			       EdgeDataCalculator.getEndY(gc, edge) 
		       ); 
		}
	

	public static int getLabelWidth(Graphics gc, ISelectableVertex vertex) {
		return gc.getFontMetrics().stringWidth(vertex.getLabel());
	}

	public static int getLabelHeight(Graphics gc, ISelectableVertex vertex) {
		return gc.getFontMetrics().getHeight();
	}
	
	public static int getLabelLeft(Graphics gc, ISelectableVertex vertex) {
		return vertex.getX() - getLabelWidth(gc, vertex) / 2;
	}

	public static int getLabelTop(Graphics gc, ISelectableVertex vertex) {
		return vertex.getY() - getLabelHeight(gc, vertex) / 2;
	}	
}