package ru.amse.smyshlyaev.grapheditor.starvertex;

import java.awt.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;


public class Child extends StarVertex {
	private boolean isSelectedVertex;
	private boolean isLocationChanging;
	
	public Child(String label, int index, 
			StarVertex father) {
		super(0, 0, father, label);
		
		isSelectedVertex = false;
		isLocationChanging = false;
		myIndex = index;

		if (father != null) {
    		father.myChildren.add(this);
		}
	}
	
	public Child(StarVertex father, Graphics gc) {
		super(0, 0, father);
		
		isSelectedVertex = false;
		isLocationChanging = false;
		myIndex = father.getChildrenNumber();

		if (father != null) {
    		father.myChildren.add(this);
		}
		
		int vertexMaximum = getMaximumRadius(gc, this);
		int centerMaximum = getMaximumWidth(gc, father);

		if (vertexMaximum / 2 > 
	        	father.getRadius() - centerMaximum / 2) {
			father.setRadius(father.getRadius()
					 + vertexMaximum
			);
		}
		
		layout(Vertex.MY_RADIUS, Vertex.MY_SPACE, gc);
	}

	private void setFatherParametres(int newX, int newY) {
		isLocationChanging = true;
		
		int x = newX - myFather.getX();
		int y = newY - myFather.getY();

		double newAngle = Math.atan2(
				y,
				x 
    	) + Math.PI;
		
		newAngle -= getAngle();

		myFather.setRadius((int) Math.sqrt(x * x + y * y));

		myFather.setStartAngle(
				myFather.getStartAngle() + newAngle
		);
		
		isLocationChanging = false;
	}

	public void setLocation(int x, int y) {
		if (isLocationChanging) {
			return;
		}

		setFatherParametres(x, y);
	}
	
	public void setX(int x) {
		setLocation(x, getY());
	}
	
	public void setY(int y) {
		setLocation(getX(), y);
	}

	public void addX(int x) {
		setX(getX() + x);
	}

	public void subX(int x) {
		setX(getX() - x);
	}
	
	public void addY(int y) {
		setY(getY() + y);
	}
	
	public void subY(int y) {
		setY(getY() - y);
	}

	public int getX() {
		int shift = (int) (myFather.getRadius() *
    		    Math.cos(getAngle()));
		
		return myFather.getX() - shift;
	}
	
	public int getY() {
		int shift = (int) (myFather.getRadius() *
    		    Math.sin(getAngle()));

		return myFather.getY() - shift;
	}

	public StarVertex getFather() {
		return myFather;
	}
	
	private double getAngle() {
		return  myFather.getStartAngle() + 
            ((myFather.getChildrenNumber() != 0) ? 
		    	  Math.PI * 2 * myIndex / 
	    	    	  myFather.getChildrenNumber()
		        	      : 0
	 	    );
	}
	public void select() {
		super.select();
		isSelectedVertex = true;
	}
	
	public void deselect() {
		super.deselect();
		isSelectedVertex = false;
	}

	public boolean shouldSelectAlone() {
		return true;
	}
	
	public boolean isSelectedVertex() {
		return isSelectedVertex;
	}
	
	public int getIndex() {
		return myIndex;
	}
	
	public boolean isChild() {
		return true;
	}
}