package ru.amse.smyshlyaev.grapheditor.starvertex;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.VertexDataCalculator;


public class StarVertex extends Vertex
                        implements ChildrenCreator {
	public static final int BUTTON_WIDTH = 10;
	public static final int BUTTON_HEIGHT = 10;	
	public final List<Child> myChildren;
	protected final StarVertex myFather;
	private int myRadius;
	private double myStartAngle;
	private int mySpace;
	protected int myIndex;
	public static final int MY_STAR_RADIUS = MY_RADIUS + 10;

	public StarVertex(int x, int y, StarVertex father,
			String label) {
		super(x, y, label);
		
		myFather = father;
		myX = x;
		myY = y;
		myChildren = new ArrayList<Child>();
		myLabel = label;
		myRadius = Vertex.MY_RADIUS / 2 + Vertex.MY_SPACE;
		myStartAngle = Math.PI / 2;
	}
	

	public StarVertex(int x, int y, StarVertex father) {
		this(x, y, father, "");
	}

	public Rectangle getChildrenCreatorRectangle(Graphics gc) {
		final int Y_SPACE = 5;

		return new Rectangle(
				getX() - BUTTON_WIDTH / 2, 
				getY() + 
			    	VertexDataCalculator.getHeight(
			    		gc, 
			    		this
			    	) / 2 - Y_SPACE - BUTTON_HEIGHT,
				BUTTON_WIDTH, 
				BUTTON_HEIGHT
		);
	}

	public int getRadius() {
		return myRadius;
	}
	
	public void setRadius(int radius) {
		myRadius = radius;
	}
	
	public double getStartAngle() {
		return myStartAngle;
	}	

	public void setStartAngle(double angle) {
		myStartAngle = angle;
	}
	
	protected int getMaximumRadius(Graphics gc, ISelectableVertex v) {
		return 2 * Math.max(
				v.getHorizontalRadius(gc),
				v.getVerticalRadius(gc)
		);
	}

	protected int getMaximumWidth(Graphics gc, ISelectableVertex v) {
		return Math.max(
				VertexDataCalculator.getWidth(gc, v),
				VertexDataCalculator.getHeight(gc, v)
		);
	}

	protected void layout(int startRadius, int startSpace,
		            	Graphics gc) {
		StarVertex child = this;
		
		while (child.myFather != null) {
		    if (child.myFather.getChildrenNumber() > 1) {
				int distanceBetweenVertices = 0;
				
				for (Child v : child.myFather.myChildren) {
					int vertexMaximum = getMaximumRadius(
							gc, 
							v
					);
					int upperMaximum = getMaximumRadius(
							gc, 
							v.myFather.getChildren().get(
								(v.myIndex == (v.myFather.getChildrenNumber() - 1)) ?
									0 : (v.myIndex + 1))
					);
					int lowerMaximum = getMaximumRadius(
							gc, 
							v.myFather.getChildren().get(
								(v.myIndex == 0) ?
									(v.myFather.getChildrenNumber() - 1) :
										(v.myIndex - 1))
					);

					if (((Math.max(
							  upperMaximum, 
							  lowerMaximum
						   ) + vertexMaximum) / 2) >
			        	distanceBetweenVertices
			        ) {
						distanceBetweenVertices = (Math.max(
								  upperMaximum, 
								  lowerMaximum
						) + vertexMaximum) / 2;
					}
				}
				
				double angle = Math.PI * 2 / 
					        	  child.myFather.getChildrenNumber();

				int suitableRadius = 
					(int) (distanceBetweenVertices *
			        	Math.cos(angle / 2) / Math.sin(angle));
				if (suitableRadius > child.myFather.getRadius()) {
					child.myFather.setRadius(suitableRadius);
				}			
		    }

		    int centerMaximum = getMaximumWidth(
					gc, 
					child.myFather
			);
		    
			int vertexMaximum = getMaximumRadius(
					gc, 
					child
			);

			if (vertexMaximum / 2 - child.getSpace() > 
		        	child.myFather.getRadius() - 
		            	centerMaximum / 2) {
				child.myFather.setRadius(
					startRadius + vertexMaximum / 2 - 
	    				child.getSpace()
				);
				child.myFather.setSpace(
					startSpace
				);
			}
			
			child = child.myFather;
		}		
	}
	
	public void createChild(Graphics gc) {
		layout(Vertex.MY_RADIUS, Vertex.MY_SPACE, gc);
	}
	
	public int getHorizontalRadius(Graphics gc) {
		int maximumChildRadius = 0;
		
		for (Child v : myChildren) {
			if (v.getHorizontalRadius(gc) > 
        			maximumChildRadius) {
				maximumChildRadius = 
					v.getHorizontalRadius(gc);
			}
		}
	
		return myRadius + maximumChildRadius + mySpace;
	}
	
	public int getVerticalRadius(Graphics gc) {
		int maximumChildRadius = 0;
	
		for (Child v : myChildren) {
			if (v.getVerticalRadius(gc) >
	    			maximumChildRadius) {
	    		maximumChildRadius = 
	    			v.getVerticalRadius(gc);
			}
		}
		
		return myRadius + maximumChildRadius + mySpace;
	}	
	
	public int getChildrenNumber() {
		return myChildren.size();
	}
	
	public int getSpace() {
		return mySpace;
	}
	
	public void setSpace(int space) {
		mySpace = space;
	}

	public List<Child> getChildren() {
		return Collections.unmodifiableList(myChildren);
	}
	
	public void setLabel(String label, Graphics gc) {
		super.setLabel(label, gc);

		if (myRadius < 
				VertexDataCalculator.getWidth(
        			gc, 
    	    		this
            	) / 2
		) {
	    	myRadius = VertexDataCalculator.getWidth(
	    			gc, 
	    			this
	    	) / 2;

	    	layout(Vertex.MY_RADIUS, Vertex.MY_SPACE, gc);		
		}
	}
	
	public boolean isParent() {
		return true;
	}
	
	public boolean removeChild(Child child) {
		if (!myChildren.contains(child)) {
			return false;
		}

		myChildren.remove(child);
		
		int counter = 0;
		
		for (Child v : myChildren) {
			if (!v.equals(child)) {
				v.myIndex = counter;
				counter += 1;
			}
		}
		
		return true;
	}
}