package ru.amse.smyshlyaev.grapheditor.ui.tool.defaulttool;

import java.awt.Graphics;
import java.awt.event.*;
import java.awt.Rectangle;
import java.util.Set;

import ru.amse.smyshlyaev.grapheditor.ui.tool.*;
import ru.amse.smyshlyaev.grapheditor.ui.JGraphComponent;
import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.ui.drawer.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.*;


/*package*/ class DefaultEdgeTool extends ToolAdapter 
                                  implements ITool {
	private final JGraphComponent myComponent;
	private int myEdgeStartX;
	private int myEdgeStartY;
	private int myEdgeEndX;
	private int myEdgeEndY;
	private ISelectableVertex myFixedVertex;
	private EdgeDrawer myEdgeDrawer;
	public static final int MY_RECTANGLE_SIDE = 10; 

	public DefaultEdgeTool(JGraphComponent component) {
		myComponent = component;
		myEdgeDrawer = new EdgeDrawer();
	}
		
	private void paintRectangle(Graphics gc, int x, int y) {
		gc.fillRect(
				x - MY_RECTANGLE_SIDE / 2, 
				y - MY_RECTANGLE_SIDE / 2, 
				MY_RECTANGLE_SIDE, 
				MY_RECTANGLE_SIDE
		);
	}
	
	public void paint(Graphics gc) {
		if (myComponent.getMarkedEdge() == null) {
			return;
		}
		
		if (myFixedVertex == null) {
			myEdgeDrawer.setEdge(myComponent.getMarkedEdge());
			
			myEdgeDrawer.draw(
					gc
			);
	
			paintRectangle(
					gc,
					EdgeDataCalculator.getStartX(gc, myComponent.getMarkedEdge()),
					EdgeDataCalculator.getStartY(gc, myComponent.getMarkedEdge())
			);

			paintRectangle(
					gc, 
					EdgeDataCalculator.getEndX(gc, myComponent.getMarkedEdge()),
					EdgeDataCalculator.getEndY(gc, myComponent.getMarkedEdge())
			);

			return;
		}
				
		gc.drawLine(
				myEdgeStartX, 
				myEdgeStartY,
				myEdgeEndX, 
				myEdgeEndY
		);
	}

	public static boolean isInRectangle(int xMiddle, 
			int yMiddle, int x, int y) {
		Rectangle rectangle = new Rectangle(
				xMiddle -
	        		MY_RECTANGLE_SIDE / 2,
	    		yMiddle - 
	        		MY_RECTANGLE_SIDE / 2,
	    		MY_RECTANGLE_SIDE,
	    		MY_RECTANGLE_SIDE
		);

		return rectangle.contains(x, y);
	}
	
	public void mousePressed(MouseEvent e) {
		IEdge myEdge = myComponent.getMarkedEdge();
		myComponent.repaint();

		if (isInRectangle(
    				EdgeDataCalculator.getStartX(myComponent.getGraphics(), myComponent.getMarkedEdge()),
					EdgeDataCalculator.getStartY(myComponent.getGraphics(), myComponent.getMarkedEdge()),
			    	e.getX(), 
			    	e.getY()
			 )
		) {
			myEdgeStartX = EdgeDataCalculator.getEndX(myComponent.getGraphics(), myComponent.getMarkedEdge());
			myEdgeStartY = EdgeDataCalculator.getEndY(myComponent.getGraphics(), myComponent.getMarkedEdge());
			myEdgeEndX = e.getX();
			myEdgeEndY = e.getY();
			myFixedVertex = myEdge.getEnd();
		}
		
		if (isInRectangle(
				EdgeDataCalculator.getEndX(myComponent.getGraphics(), myComponent.getMarkedEdge()),
				EdgeDataCalculator.getEndY(myComponent.getGraphics(), myComponent.getMarkedEdge()),
				e.getX(),
				e.getY()
			 )
		) {
			myEdgeStartX = EdgeDataCalculator.getStartX(myComponent.getGraphics(), myComponent.getMarkedEdge());
			myEdgeStartY = EdgeDataCalculator.getStartY(myComponent.getGraphics(), myComponent.getMarkedEdge());
			myEdgeEndX = e.getX();
			myEdgeEndY = e.getY();
			myFixedVertex = myEdge.getStart();
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (myFixedVertex == null) {
			return;
		}
		
		IEdge myEdge = myComponent.getMarkedEdge();
		
		Set<ISelectableVertex> vertices = 
	    		myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			if (VertexDataCalculator.isInVertex(
					myComponent.getGraphics(),
					v,
					myEdgeEndX, 
					myEdgeEndY
				) && (!myFixedVertex.equals(v))) {
				if (myFixedVertex.equals(myEdge.getStart())) {
					if (!myFixedVertex.hasEdge(
							new Edge(myFixedVertex, v)
						)
					) {
    					myEdge.setEnd(v);
					}
				} else {
					if (!v.hasEdge(
							new Edge(v, myFixedVertex)
						)
					) {
    					myEdge.setStart(v);	
					}
				}
				
				break;
			}
		}
		
		myFixedVertex = null;
		
		myComponent.repaint();
	}

	public void mouseDragged(MouseEvent e) {
		if (myFixedVertex == null) {
			return;
		}
		
		myEdgeEndX = e.getX();
		myEdgeEndY = e.getY();
		
		myComponent.repaint();
	}
}