package ru.amse.smyshlyaev.grapheditor.ui.tool.defaulttool;

import java.awt.event.MouseEvent;
import java.util.Set;
import java.awt.*;

import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.ui.JGraphComponent;
import ru.amse.smyshlyaev.grapheditor.ui.tool.ITool;


/*package*/ class DefaultRectangleTool extends VertexMarker
                      implements ITool {
	private int myStartX;
	private int myStartY;
	private int myLeft;
	private int myRight;
	private int myTop;
	private int myBottom;
	private boolean drawRectangle;

	public DefaultRectangleTool(JGraphComponent component) {
		super(component);
		drawRectangle = true;
	}

	public void paint(Graphics gc) {
		if (drawRectangle) {
			Graphics2D g2d = (Graphics2D) gc;
	
			float[] dash = {4.0f, 1.0f};
			
			Stroke stroke = g2d.getStroke();

			g2d.setStroke(
					new BasicStroke(
							1.0f,
                            BasicStroke.CAP_BUTT,
                            BasicStroke.JOIN_MITER,
                            1.0f,
                            dash,
                            0
                    )
			);
			
			g2d.drawRect(
					myLeft,
					myTop,
					myRight - myLeft,
					myBottom - myTop
	    	);
			
			g2d.setStroke(stroke);
			
			return;
		}

		super.paint(gc);
	}

	public void mousePressed(MouseEvent e) {
		drawRectangle = true;

		myStartX = e.getX();
		myStartY = e.getY();
		
		myLeft = myStartX;
		myRight = 0;
		myTop = myStartY;
		myBottom = 0;

		myComponent.repaint();
	}

	public void mouseReleased(MouseEvent e) {
		drawRectangle = false;
		
		Set<ISelectableVertex> vertices = 
			myComponent.getGraph().getVertices();

		Rectangle rectangle = new Rectangle(
				myLeft,
				myTop,
				myRight - myLeft,
				myBottom - myTop
		);
		
		Rectangle vertexRectangle = new Rectangle();
		
		for (ISelectableVertex v : vertices) {
			int width = 2 * v.getHorizontalRadius(myComponent.getGraphics());
			int height = 2 * v.getVerticalRadius(myComponent.getGraphics());
				
			vertexRectangle.setRect(
					v.getX() - width / 2,
					v.getY() - height / 2,
		        	width,
		        	height
			);
			
			if (rectangle.contains(vertexRectangle)
				&& !v.shouldSelectAlone()) {
					myComponent.addMarkedVertex(v);
			}
		}

		myComponent.repaint();
	}
		
	public void mouseDragged(MouseEvent e){
		drawRectangle = true;

		int x = e.getX();
		int y = e.getY();
		
		if (myStartX < x) {
			myLeft = myStartX;
			myRight = x;
		} else {
			myLeft = x;
			myRight = myStartX;
		}
		
		if (myStartY < y) {
			myTop = myStartY;
			myBottom = y;
		} else {
			myTop = y;
			myBottom = myStartY;
		}

    	myComponent.repaint();
	}
}