package ru.amse.smyshlyaev.grapheditor.ui.tool.edgecreator;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import ru.amse.smyshlyaev.grapheditor.ui.tool.*;
import ru.amse.smyshlyaev.grapheditor.graphdatacalculator.*;
import ru.amse.smyshlyaev.grapheditor.graph.*;
import ru.amse.smyshlyaev.grapheditor.ui.*;


public class EdgeCreator extends ToolAdapter 
                         implements ITool {
	private final JGraphComponent myComponent;
	private ISelectableVertex vertex;
	private int myEndX;
	private int myEndY;
	public static final int POINT_RADIUS = 4;

	public EdgeCreator(JGraphComponent component) {
		myComponent = component;
		vertex = null;
	}
		
	public void paint(Graphics gc) {
		Set<ISelectableVertex> vertices = 
	    		myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			gc.fillOval(
				v.getX() - POINT_RADIUS,
				v.getY() - 
		    		VertexDataCalculator.getHeight(gc, v) / 2,
		    	2 * POINT_RADIUS,
		    	2 * POINT_RADIUS
			);
		}
		
		if (vertex == null) {
			return;
		}

		gc.drawLine(
				vertex.getX(), 
				vertex.getY() -
			    	VertexDataCalculator.getHeight(
			    		gc, 
			    		vertex
			    	) / 2 + POINT_RADIUS / 2,
				myEndX,
				myEndY
		);
	}

	private boolean isInVertexPoint(Graphics gc, 
		            	ISelectableVertex vertex, int x, int y) {
		int shiftedX = x - vertex.getX();
		int shiftedY = y - vertex.getY() + 
		      		VertexDataCalculator.getHeight(
		          		gc,
		          		vertex
		       		) / 2 - POINT_RADIUS / 2; 

		return (shiftedX * shiftedX + shiftedY * shiftedY) <= 
			POINT_RADIUS * POINT_RADIUS;
	}
	
	public void mousePressed(MouseEvent e) {
		Set<ISelectableVertex> vertices =
    			myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			if (isInVertexPoint(
					myComponent.getGraphics(),
					v,
					e.getX(), 
					e.getY()
				)
			) {
				vertex = v;
				return;
			}
		}
	}

	public void mouseReleased(MouseEvent e) {
		if (vertex == null) {
			return;
		}
		
		Set<ISelectableVertex> vertices =
	    		myComponent.getGraph().getVertices();
		
		for (ISelectableVertex v : vertices) {
			if (VertexDataCalculator.isInVertex(
					myComponent.getGraphics(), 
					v, 
					e.getX(), 
					e.getY()
				)
			) {
				myComponent.addEdge(
						new Edge(vertex, v)
				);
				
				break;
			}
		}
		
		vertex = null;
		
		myComponent.repaint();
	}

	public void mouseDragged(MouseEvent e) {
		if (vertex == null) {
			return;
		}
		
		myEndX = e.getX();
		myEndY = e.getY();
		
		myComponent.repaint();
	}
}